const MIN_TOUCHMOVE_DISTANCE_SQUARED = 49;

function TouchEventDemoController() {
    this.startPositions = new Object();

    // Don't allow the document to pan on iPhone OS
    document.addEventListener('touchmove', function(e) { e.preventDefault(); }, false);
}

TouchEventDemoController.prototype.distanceSquared = function(x1, y1, x2, y2) {
    return ((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
}

// Sets and then quickly removes a 'highlighted' class an element. 
// CSS Transitions will handle the actual animation.
TouchEventDemoController.prototype.flashEvent = function(id) {
    var element = document.getElementById(id);
    element.className = 'highlighted';
    window.setTimeout(function() { 
        element.className = ''; 
    }, 0);
}

TouchEventDemoController.prototype.registerHandler = function(eventName) {
    // If we pass in an object instead of a method as addEventListener's second parameter,
    // when the event is dispatched a method named 'handleEvent' will be called on the object.
    // 
    // Mouse events are not dispatched to the document node, so use a hidden element that
    // overlays the whole page instead.
    document.getElementById('eventtarget').addEventListener(eventName, this, false);
}

TouchEventDemoController.prototype.handleEvent = function(event) {
    this[event.type](event);
}

/******************************************************************************
  Event Handlers 
******************************************************************************/

TouchEventDemoController.prototype.mousedown = function(event) {
  this.flashEvent('mousedown');
}

TouchEventDemoController.prototype.mousemove = function(event) {
  this.flashEvent('mousemove');
}

TouchEventDemoController.prototype.mouseup = function(event) {
  this.flashEvent('mouseup');
}

TouchEventDemoController.prototype.touchstart = function(event) {
  for (var i = 0; i < event.changedTouches.length; i++) {
    var touch = event.changedTouches[i];
    this.startPositions[touch.identifier] = { x: touch.pageX, y: touch.pageY };
  }
  
  this.flashEvent('touchstart');
}

TouchEventDemoController.prototype.touchmove = function(event) {
  // To make demonstrations easier, suppress touchmove events until a finger has moved more than
  // a few pixels away from where the touch started.
  for (var i = 0; i < event.changedTouches.length; i++) {
    var touch = event.changedTouches[i];
    startPosition = this.startPositions[touch.identifier]
    if (startPosition && this.distanceSquared(touch.pageX, touch.pageY, startPosition.x, startPosition.y) < MIN_TOUCHMOVE_DISTANCE_SQUARED)
      continue;
    
    delete this.startPositions[touch.identifier];
    this.flashEvent('touchmove');
  }
}

TouchEventDemoController.prototype.touchend = function(event) {
  for (var i = 0; i < event.changedTouches.length; i++)
    delete this.startPositions[event.changedTouches[i].identifier];
  
  this.flashEvent('touchend');
}

TouchEventDemoController.prototype.touchcancel = function(event) {
  this.startPositions = new Object();
  this.flashEvent('touchcancel');
}

TouchEventDemoController.prototype.gesturestart = function(event) {
  this.flashEvent('gesturestart');
}

TouchEventDemoController.prototype.gesturechange = function(event) {
  this.flashEvent('gesturechange');
}

TouchEventDemoController.prototype.gestureend = function(event) {
  this.flashEvent('gestureend');
}
